"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.existVariableInTestName = void 0;
var util_1 = require("./util");
exports.existVariableInTestName = {
    meta: {
        fixable: 'code',
        messages: {
            'UndefinedVariables': 'The test name contains a variable that is not defined.'
        }
    },
    create: function (context) {
        var sourceCode = context.getSourceCode();
        return {
            TaggedTemplateExpression: function (node) {
                var _a, _b;
                var results = (0, util_1.getTestEachVariables)(node, sourceCode);
                if (results === undefined) {
                    return;
                }
                var header = results.header;
                // 親の引数に第1引数にLiteralがなければ終了
                var parent = node.parent;
                if (parent.type !== 'CallExpression' || parent.arguments.length !== 2) {
                    return;
                }
                if (parent.arguments[0].type !== 'Literal') {
                    return;
                }
                // テスト名から、使用している引数の一覧取得
                var testName = parent.arguments[0].value;
                var matchVariables = (_b = (_a = testName.match(/\$\w*/g)) === null || _a === void 0 ? void 0 : _a.map(function (m) { return m.replace('$', ''); })) !== null && _b !== void 0 ? _b : [];
                // 使用している引数が、テンプレート文字列に存在するかチェック
                var isNotExist = matchVariables.map(function (m) { return header.includes(m); }).includes(false);
                if (isNotExist) {
                    context.report({ node: parent.arguments[0], messageId: 'UndefinedVariables' });
                }
            }
        };
    }
};
