"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.format = exports.defaultOptions = void 0;
var os_1 = __importDefault(require("os"));
var meaw_1 = require("meaw");
var util_1 = require("./util");
exports.defaultOptions = {
    lineBreakStyle: os_1.default.EOL === '\n' ? 'unix' : 'windows',
    indent: 4,
    eastAsianWidth: {}
};
exports.format = {
    meta: {
        fixable: 'code',
        messages: {
            'TestEachArgumentMismatch': 'The variable specified in the header of test.each does not match the number of arguments inserted.',
            'TestEachArgumentWidthMismatch': 'Element widths are not aligned.'
        },
        schema: [
            {
                type: 'object',
                properties: {
                    lineBreakStyle: {
                        enum: ['unix', 'windows'],
                        default: exports.defaultOptions.lineBreakStyle,
                        optional: true
                    },
                    indent: {
                        type: 'number',
                        default: 4,
                        optional: true
                    },
                    eastAsianWidth: {
                        type: 'object',
                        properties: {
                            N: { type: 'number', description: 'Neutral', optional: true },
                            Na: { type: 'number', description: 'Narrow', optional: true },
                            W: { type: 'number', description: 'Wide', optional: true },
                            F: { type: 'number', description: 'Full Width', optional: true },
                            H: { type: 'number', description: 'Half Width', optional: true },
                            A: { type: 'number', description: 'Ambiguous', optional: true }
                        },
                        optional: true
                    }
                }
            }
        ]
    },
    create: function (context) {
        var _a, _b, _c, _d;
        var sourceCode = context.getSourceCode();
        var options = context.options.length === 0 ? exports.defaultOptions : (_a = context.options[0]) !== null && _a !== void 0 ? _a : exports.defaultOptions;
        var eol = ((_b = options.lineBreakStyle) !== null && _b !== void 0 ? _b : exports.defaultOptions.lineBreakStyle) === 'windows' ? '\r\n' : '\n';
        var indent = (_c = options.indent) !== null && _c !== void 0 ? _c : exports.defaultOptions.indent;
        var eastAsianWidth = __assign(__assign({}, exports.defaultOptions.eastAsianWidth), ((_d = options.eastAsianWidth) !== null && _d !== void 0 ? _d : {}));
        return {
            TaggedTemplateExpression: function (node) {
                var results = (0, util_1.getTestEachVariables)(node, sourceCode);
                if (results === undefined) {
                    return;
                }
                var range = results.range, loc = results.loc, source = results.source, header = results.header, splitCharCount = results.splitCharCount;
                // テンプレートに指定している引数の区切り文字数がヘッダーの引く1つ少ない数の倍数でなければ終了
                if (splitCharCount % (header.length - 1) !== 0) {
                    context.report({ node: node, messageId: 'TestEachArgumentMismatch' });
                    return;
                }
                // 引数を取得
                var argument = node.quasi.expressions.map(function (m) {
                    var mRange = m.range;
                    return sourceCode.getText(node, range[0] - mRange[0], mRange[1] - range[1]);
                });
                // 引数の数がヘッダーの倍数でなければ終了
                if (argument.length === 0 || argument.length % header.length !== 0) {
                    context.report({ node: node, messageId: 'TestEachArgumentMismatch' });
                    return;
                }
                // reduceで利用する指定した分割数で配列を分割する関数
                var mapFnChunk = function (p, c, i, chunkSize) {
                    var chunkIndex = Math.floor(i / chunkSize);
                    if (!p[chunkIndex]) {
                        p[chunkIndex] = [];
                    }
                    p[chunkIndex].push(c);
                    return p;
                };
                /**
                 * 文字列とインデックスと元の配列を引数に取り、文字列を返す関数
                 * @param element   ベースの文字列
                 * @param index     ベースの文字列が格納されているインデックス
                 * @param array     ベースの文字列が格納されている配列
                 * @param maxLength インデックス番号に対応する最大文字数
                 * @returns
                 */
                var mapFn = function (element, index, array, maxLength) {
                    if (index === 0) {
                        return element + ' '.repeat(maxLength - (0, meaw_1.computeWidth)(element, eastAsianWidth));
                    }
                    else if (index === array.length - 1) {
                        return element;
                    }
                    return element + ' '.repeat(maxLength - (0, meaw_1.computeWidth)(element, eastAsianWidth));
                };
                // 表のカラムごとの最大文字数を取得
                var maxLengthArray = __spreadArray(__spreadArray([], header.map(function (m) { return (0, meaw_1.computeWidth)(m, eastAsianWidth); }), true), argument.map(function (m) { return (0, meaw_1.computeWidth)(m, eastAsianWidth) + 3; }), true).reduce(function (p, c, i) { return mapFnChunk(p, c, i, header.length); }, [])
                    // 2重配列のIndex番号ごとの最大値を取得
                    .reduce(function (p, c, i) {
                    if (i === 0) {
                        return c;
                    }
                    return p.map(function (m, i) { return Math.max(m, c[i]); });
                }, []);
                // ベースインデントの個数を取得
                var startString = sourceCode.getText().split(eol)[loc.start.line - 1];
                var multiple = (startString.length - startString.trimStart().length) / indent;
                var startStringMatchHeader = source.split(eol)[0].indexOf(header[0]);
                var newNodeText = startStringMatchHeader === -1 ?
                    source.split(eol)[0] + eol :
                    source.split(eol)[0].slice(0, startStringMatchHeader) + eol;
                // ヘッダーを追加
                newNodeText += ' '.repeat(indent * (multiple + 1)) + header.map(function (m, i, s) { return mapFn(m, i, s, maxLengthArray[i]); }).join(' | ') + eol;
                // 引数を追加
                newNodeText += argument
                    .reduce(function (p, c, i) { return mapFnChunk(p, c, i, header.length); }, [])
                    .map(function (m) { return ' '.repeat(indent * (multiple + 1)) + m.map(function (m, i, s) { return mapFn("${".concat(m, "}"), i, s, maxLengthArray[i]); }).join(' | '); })
                    .join(eol) + eol;
                // エンドインデントを追加
                newNodeText += ' '.repeat(indent * multiple) + '`';
                // 生成した文字列と元の文字列が異なる場合は修正
                if (source !== newNodeText) {
                    context.report({
                        node: node,
                        messageId: 'TestEachArgumentWidthMismatch',
                        fix: function (fixer) { return fixer.replaceText(node, newNodeText); }
                    });
                }
            }
        };
    }
};
